/**
 * 				CORE FUNCTION
 * 					versione 1.0
 * 					Designed by: BGG
 * 
 */
package com.bgg.sfix.LogicApp;

import java.io.BufferedInputStream;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.net.URL;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import java.util.ArrayList;
import java.util.Locale;

import android.content.Context;
import android.content.Intent;
import android.net.Uri;
import android.os.Environment;
import android.util.Base64;
import android.util.Log;

import com.bgg.sfix.LogicApp.FixAudioFile.FIELD_tag;
import com.gracenote.mmid.MobileSDK.GNConfig;
import com.gracenote.mmid.MobileSDK.GNFingerprintResult;
import com.gracenote.mmid.MobileSDK.GNFingerprintResultReady;
import com.gracenote.mmid.MobileSDK.GNOperations;
import com.gracenote.mmid.MobileSDK.GNSearchResponse;
import com.gracenote.mmid.MobileSDK.GNSearchResult;
import com.gracenote.mmid.MobileSDK.GNSearchResultReady;

public class CoreFunctions {
	/**
	 * Metodo che indica se il file  da analizzare oppure  da saltare.
	 * Praticamente serve per saltare tutti quei files che nella directory non sono file audio.
	 * Nell'essenza questo algoritmo di identificazione  molto semplice e rozzo, in futuro, quando avremmo 
	 * risolto ben pi gravi problemi, si pu facilmente rifinire con librerie apposite.
	 * Per il momento la verifica viene fatta sull'estensione.
	 * Si prendono gli ultimi 3 caratteri del nome del file da analizzare e si confronta con
	 * l'estensioni dei files musicali conosciuti.
	 * @param pfile [in]	File da identificare se analizzabile oppure no
	 * @return 'true' in caso il file sia un file analizzabile (una traccia audio) oppure 'false' altrimenti.
	 */
	public static boolean IsFileAnalizable(File pfile){
		if(pfile==null) return false;
		if(pfile.isFile()==false) return true;
		String namefile=pfile.getName();
		String extens = namefile.substring(namefile.length()-3, namefile.length());
		extens=extens.toLowerCase(Locale.getDefault());

		/*Verifica identit MP3*/
		if(extens.compareTo("mp3")==0){
			return true;
		}			
		return false;
	}
	
	
	/**
	 * Esegue un calgolo dei file che devono essere analizzati in una directory.
	 * Serve all'algoritmo per avere una stima dei files che dovranno essere processati.
	 * @param _dfile [in]	class Java File (un file o una directory) da analizzare
	 * @return il numero di file che devono essere analizzati dall'algortimo
	 */
	public static int GetNumberFiles(File _dfile, OptionAlgorithm _option){
		if(_dfile.isFile()==true){
			if(CoreFunctions.IsFileAnalizable(_dfile)==true) return 1;
			else return 0;
		}
		
		/*Qui non c' molto da dire  una funzione ricorsiva.
		 * Prendo la lista di files contenuti nella cartella passata e ognuno vedo se  analizzabile
		 * se  una cartella invoco questa stessa funzione (RICORSIVITA') in caso, ovviamente, l'opzione
		 * RICORSIVITA' delle opzioni  attiva*/
		File[] list_file = _dfile.listFiles();
		if(list_file==null) return 0;
		int count=0;
		for(int i=0; i<list_file.length; i++){
			if(list_file[i].isFile()==false){
				if(_option.ricorsive==true)
					count+=CoreFunctions.GetNumberFiles(list_file[i],_option);
			}else{
				if(CoreFunctions.IsFileAnalizable(list_file[i])){
					count+=1;
				}
			}
			
		}
		return count;
	}
	
	/**
	 * Inizializza la libreria di graceNote. Deve essere invocata prima di richiamare gli altri
	 * metodi di GraceNote
	 * @param _contesto	[in]	Contesto applicativo. Passare ad Es. 'getApplicationContext()'
	 * @throws Exception	In Caso di errore nell'inizializzazione
	 */
	public static void InitGraceNote_Library(Context _contesto, OptionAlgorithm _option) throws Exception{
		if(CoreFunctions.serverGraceNote == null){
			CoreFunctions.serverGraceNote = GNConfig.init(CoreFunctions.ID_GRACENOTE_APP, _contesto);
			if(CoreFunctions.serverGraceNote == null){
				throw new Exception("Impossibile stabilire un collegamento con il server!");
			}
			if(_option.downloadCovert==true){
				CoreFunctions.serverGraceNote.setProperty("content.coverArt", "1");
			}else{
				CoreFunctions.serverGraceNote.setProperty("content.coverArt", "0");
			}
		}
	}
	
	/**
	 * Effettua l'upload di una traccia audio e ne ritora un risultato da dove  possibile estrarre
	 * i tags informativi (es. Titolo, Artista, ...)
	 * @param _pFile [in]		File di cui fare il fingerprinting
	 * @return					Un risulato di ricerca.
	 * @throws Exception		In caso di errore. Lancia un'eccezione anche se il file non viene riconosciuto
	 * 							(no match found).
	 * NOTABENE: Questo metodo  BLOCCANTE. Dovrebbe essere lanciato in un thread apparte.
	 */
	public static RisultatoRicerca UpDateTrack_forFingerPrinting(File _pFile) throws Exception{
		if(CoreFunctions.serverGraceNote==null){
			throw new Exception("Impossibile utilizzare la funzione di fingerprinting senza prima aver " +
					"effettuato l'inizializzazione delle librerie!");
		}
		CoreFunctions.RisultatoAnalisi_FingerPrinting signatureAudio = new CoreFunctions.RisultatoAnalisi_FingerPrinting();
		RisultatoRicerca rts = new RisultatoRicerca();
		
		GNOperations.fingerprintMIDFileFromFile(signatureAudio,CoreFunctions.serverGraceNote, _pFile.getAbsolutePath());
		while(!signatureAudio.Result_IsReady());
		GNOperations.searchByFingerprint(rts, CoreFunctions.serverGraceNote, signatureAudio.GetData());
		while(!rts.Result_IsReady());
		return rts;
	}
	
	
	
	/**
	 * Analizza e fixa un file secondo i settaggi delle opzioni passate come parametro.
	 * NOTABENE: Questo metodo  bloccante!
	 * @param _option [in]		Opzioni di elaborazione
	 * @param _file [in]		File da elaborare
	 * @return 'true' se il file viene analizzato correttamente, altrimenti 'false' se il file viene saltato poich analizzato di recente
	 * @throws Exception		In caso di qualsiasi errore!
	 * 							-Impossibile accesso al file (inesistenza o non permessi di lettura/scrittura)
	 * 							-Nessuna corrispondenza trovata
	 */
	public static boolean AnalizeAndFix_File(OptionAlgorithm _option, File _file) throws Exception{
		Log.d("@CoreInformation", "Analisi file musicale: " + _file.getAbsolutePath());
		if(_file.isFile()==false || CoreFunctions.IsFileAnalizable(_file)==false) throw new Exception("Tentativo di analisi di un file non decodificabile!");
		
		if(CoreFunctions.registro!=null){
			if(CoreFunctions.registro.File_IsRecentAnalize(_option, _file)==true){
				if(_option.modFileSystem==true){
					_file = CoreFunctions.OrganizeAudioFile_inFileSystem(new FixAudioFile(_file.getAbsolutePath()));
					CoreFunctions.registro.InsertTrackFixed(_file, _option);
				}
				return false;
			}
		}
		
		FixAudioFile afile=new FixAudioFile(_file.getAbsolutePath());
		
		switch(_option.methodFix){
		case SCANSIONE_COMPLETA:
			final int MAX_MATCH_FOR_QUERY_TEXT = 4;
			ArrayList<FIELD_tag> miss_tags = CoreFunctions.GetMissTags(afile);
			if(miss_tags.size()==0) break;
			RisultatoRicerca result_query = null;
			if(miss_tags.contains(FIELD_tag.TAG_TITOLO)==true || miss_tags.contains(FIELD_tag.TAG_ARTISTA)==true){
				//se manca il titolo e/o l'artista allora faccio la ricerca fingerprinting
				result_query = CoreFunctions.UpDateTrack_forFingerPrinting(_file);
			}else{
				//faccio ricerca testuale
				result_query = CoreFunctions.SearchTrack_fromTags(afile);
				if(result_query.Get_nResponses() > MAX_MATCH_FOR_QUERY_TEXT){
					//con la ricerca testuale ho ottenuto troppi risultati. devo comunque fare fingerprinting
					result_query = CoreFunctions.UpDateTrack_forFingerPrinting(_file);
				}
			}
			for(FIELD_tag xtag:miss_tags){
				//per ogni tag mancante lo ricavo dal risultato della query al database ottenuta
				try{
					if(!(xtag.compareTo(FIELD_tag.TAG_COVER_IMAGE)==0 && _option.downloadCovert==false)){
						afile.SetATag(xtag, result_query.GetGenericTag(xtag), result_query.GetCoverAlbumMIMEtype());
					}
				}catch(Exception ex){
					//in caso non c' copertina
					Log.e("@sFix:CoreFunctions", ex.getMessage());
				}
			}
			//salvo tutti i nuovi file
			afile.Commit();
			break;
			
			
			
		case SCANSIONE_DEBUG:
			throw new Exception("Meotdo algoritmo non disponibile in questa versione del software!");
			
			
			
			
		case SCANSIONE_ESTESA:
			throw new Exception("Meotdo algoritmo non ancora implementato!");
			
			
			
			
		case SCANSIONE_RAPIDA:
			throw new Exception("Meotdo algoritmo non ancora implementato!");
			
			
			
		default:
			throw new Exception("Impossibile determinare il metodo di Fixing");
		}
		
		if(_option.modFileSystem==true){
			_file=CoreFunctions.OrganizeAudioFile_inFileSystem(afile);
			afile=null;		//!!<----- CANCELLO L'OGGETTO MUSICALE
		}
		
		if(CoreFunctions.registro!=null)
			CoreFunctions.registro.InsertTrackFixed(_file, _option);
		
		return true;
	}
	
	
	/**
	 * Lancia il MEDIA_SCAN di android in modo da aggiornare le modifiche fatte sul filesystem.
	 * (Andrebbe invocato alla fine del processo)
	 * @param contesto [in]		Contesto dell'applicazione
	 */
	public static void LaunchMediaScan(Context contesto){
		contesto.sendBroadcast(new Intent(Intent.ACTION_MEDIA_MOUNTED, Uri.parse("file://"
	            + Environment.getExternalStorageDirectory())));
	}
	
	
	/**
	 * Inizializza il registro dell'applicazione
	 * @param contesto [in]		Contesto dell'applicazione
	 * @throws Exception In caso non si riesca ad eccere alla memoria di massa
	 */
	public static void InitRegisterFix(Context contesto) throws Exception{
		CoreFunctions.registro= new RegisterFix(contesto);
	}

	
	
	/**
	 * Ritorna il miglior risultato che corrisponde all'audio file passato nel parametro.
	 * La ricerca viene effettuata in base al titolo, artista ed album della canzone passata nel parametro.
	 * Non sono necessari tutti e 3 i tags, ma pi informazioni corrispondono ad un criterio di ricerca migliore.
	 * @param afile [in]	Il file audio per recuperare i tags da valutare
	 * @return Il risultato corrispondente alla canzone che meglio matcha i tags della canzone nel parametro
	 * @throws Exception In caso non inizializzazioni della libreria di GN o corruzioni del file audio.
	 */
	public static RisultatoRicerca SearchTrack_fromTags(FixAudioFile afile) throws Exception{
		if(CoreFunctions.serverGraceNote==null){
			throw new Exception("Impossibile utilizzare la funzione di fingerprinting senza prima aver " +
					"effettuato l'inizializzazione delle librerie!");
		}
		if(afile==null){
			throw new Exception("Impossibile accedere alle informazioni multimediali del file audio!");
		}
		
		RisultatoRicerca rts= new RisultatoRicerca();
		GNOperations.searchByText(rts, CoreFunctions.serverGraceNote, afile.GetArtist(), afile.GetAlbum(), afile.GetTitle());
		while(!rts.Result_IsReady());
		return rts;
	}
	
	
	
	/**
	 * Effettua una modifica del file system organizzandolo secondo uno schema riferito al brano musicale.
	 * In poche parole, dato un brano musicale questa funzione sposta il file fisico in una cartella che segue
	 * un l'ordine stabilito (es. "../music/artista/album/titolo.mp3").
	 * Rinominando eventualmente anche il file fisico. 
	 * Se la cartella non esiste viene creata!
	 * @param _afile [inout] File audio di riferimento, da spostare e sistemare nel filesystem
	 * @return Il nuovo riferimento al file spostato nel filesistem!
	 * @throws Exception in caso di errore nello spostamento
	 * @note NOTA BENE: il parametro passato alla funzione 'FixAudioFile' verr corrotto!
	 * 		Non  garantita ASSOLUTAMENTE l'integrit dell'oggetto della classe una volta terminata questa procedura.
	 * 		E' possibile comunque ritonnere conformit all'oggetto riallocandolo costruendolo col nuovo file
	 * 		in return da questa stessa funzione.
	 * 		Non viene fatto direttamente all'interno di questa procedura per non aggravare il costo computazionale
	 * 		della stessa
	 */
	public static File OrganizeAudioFile_inFileSystem(FixAudioFile _afile) throws Exception{
		if(_afile==null) throw new Exception("Impossibile accedere al file audio");
		File file_current = new File(_afile.GetAbsoluteFileName());
		if(file_current.exists()==false){
			throw new Exception("Impossibile accedere al file audio:" + _afile.GetAbsoluteFileName());
		}
		
		String artist = _afile.GetArtist();
			artist = artist.replace(':', ' ');
			artist = artist.replace('\\', ' ');
			artist = artist.replace('/', ' ');
			artist = artist.replace('?', ' ');
			artist = artist.replace('*', ' ');
			artist = artist.replace('<', ' ');
			artist = artist.replace('>', ' ');
			artist = artist.replace('|', ' ');
			artist = artist.replace('"', ' ');
		String album = _afile.GetAlbum();
			album = album.replace(':', ' ');
			album = album.replace('\\', ' ');
			album = album.replace('/', ' ');
			album = album.replace('?', ' ');
			album = album.replace('*', ' ');
			album = album.replace('<', ' ');
			album = album.replace('>', ' ');
			album = album.replace('|', ' ');
			album = album.replace('"', ' ');
		String title = _afile.GetTitle();
			title = title.replace(':', ' ');
			title = title.replace('\\', ' ');
			title = title.replace('/', ' ');
			title = title.replace('?', ' ');
			title = title.replace('*', ' ');
			title = title.replace('<', ' ');
			title = title.replace('>', ' ');
			title = title.replace('|', ' ');
			title = title.replace('"', ' ');
		
		/*FASE 1) SPOSTO I FILE NELLA CARTELLA MUSICA DEL SISTEMA SECONDO LA REGOLA ".../music/artist/album/file.mp3"*/
		String potenzial_directory = Environment.getExternalStoragePublicDirectory(Environment.DIRECTORY_MUSIC).getAbsolutePath();
			potenzial_directory+="/"+artist+"/"+album+"/";
		File after_file = new File(potenzial_directory + file_current.getName());
		
		//verifico se il file si trova gi nella directory fixata
		if(file_current.getAbsolutePath().compareTo(after_file.getAbsolutePath())!=0){
			//il file non si trova. quindi creo la nuova directory di destinazione se non esiste
			if(after_file.getParentFile().exists()==false)
				after_file.getParentFile().mkdirs();
			//sposto il file
			file_current.renameTo(after_file);
			//ricalcolo il riferimento del file corrente
			file_current = after_file;
			//controllo che l'operazione sia andata a buon fine
			if(file_current.exists()==false)
				throw new Exception("Impossibile spostare il file: "+_afile.GetAbsoluteFileName());
		}
		
		
		/* FASE 2) CAMBIO IL NOME DEL FILE COME DA TITOLO*/
		after_file = new File(file_current.getParentFile().getAbsolutePath() + "/" + title + CoreFunctions.GetExention_fromFile(file_current));
		if(file_current.getAbsolutePath().compareTo(after_file.getAbsolutePath())!=0){
			//il nome del file  diverso da quello che dovrebbe avere (titolo) allora rinomino
			file_current.renameTo(after_file);
			//ricalcolo il riferimento del file corrente
			file_current = after_file;
			//controllo che l'operazione sia andata a buon fine
			if(file_current.exists()==false)
				throw new Exception("Impossibile rinominare il file: "+_afile.GetAbsoluteFileName());
		}
		
		return file_current;
	}
	
	
	/**
	 * Ritorna tutti i tag che mancano al brano musicale passato in input come parametro.
	 * @param afile [in]	Il file musicale che si vuole verificare
	 * @return Tutti i tag che mancano al brano passato come input. 
	 * 			Ritorna una lista a dimensione 0 in caso il brano sia completamente taggato.
	 * @note Il primo elemento della lista, se manca,  sempre la copertina dell'album!
	 * 		Questo per ragioni di scadenze temporali nel download
	 */
	public static ArrayList<FixAudioFile.FIELD_tag> GetMissTags(final FixAudioFile afile){
		ArrayList<FixAudioFile.FIELD_tag> rts = new ArrayList<FixAudioFile.FIELD_tag>();
		if(afile==null) return rts;
		if(afile.GetCoverAlbum()==null)
			rts.add(FIELD_tag.TAG_COVER_IMAGE);
		if(afile.GetTitle().isEmpty())
			rts.add(FIELD_tag.TAG_TITOLO);
		if(afile.GetArtist().isEmpty())
			rts.add(FIELD_tag.TAG_ARTISTA);
		if(afile.GetAlbum().isEmpty())
			rts.add(FIELD_tag.TAG_ALBUM);
		if(afile.GetAlbumArtist().isEmpty());
			rts.add(FIELD_tag.TAG_ARTISTA_ALBUM);
		if(afile.GetYear().isEmpty())
			rts.add(FIELD_tag.TAG_ANNO);
		if(afile.GetGenre()==0)
			rts.add(FIELD_tag.TAG_NTRACK);
		return rts;
	}
	
	
	
	/**
	 * Ritorna l'estensione di un file. 
	 * 	Ad esempio: per il file 'musica.mp3' ritorna '.mp3'
	 * NOTA BENE: ritorna anche il punto!
	 * @param _pfile [in]	Il file di cui si vuole conoscere l'estensione
	 * @return la stringa rappresentativa dell'estensione. In caso non sia presente ritorna una stringa vuota
	 */
	public static String GetExention_fromFile(final File _pfile){
		if(_pfile==null) return new String("");
		String strnamefile = _pfile.getName();
		int offset = strnamefile.lastIndexOf('.');
		if(offset > 0){
			return strnamefile.substring(offset);
		}
		return new String("");
	}
	
	
	
	/**
	 * Effettua il calcolo della signature (firma) di un file (tramite l'hash MD5)
	 * @param _pfile [in]	File di cui si vuole calcolare la firma
	 * @return L'hash (signature) del files
	 * @throws NoSuchAlgorithmException In caso l'MD5 non sia presente sull'architettura in elaborazione
	 * @throws IOException In caso di non accesso al file in lettura
	 */
	public static String GetSignatureOfFile(final File _pfile) throws NoSuchAlgorithmException, IOException{
		if(_pfile==null) return new String("");
		
		final char[] hexDigits = "0123456789abcdef".toCharArray();
		byte[] bytes = new byte[4096];
		int read = 0; 
		MessageDigest digest = MessageDigest.getInstance("MD5");
		InputStream is = new FileInputStream(_pfile);
		while ((read = is.read(bytes)) != -1) {
             digest.update(bytes, 0, read);
         }
		
		byte[] messageDigest = digest.digest();
		 
        StringBuilder sb = new StringBuilder(32);

        for (byte b : messageDigest) {
            sb.append(hexDigits[(b >> 4) & 0x0f]);
            sb.append(hexDigits[b & 0x0f]);
        }
		
        is.close();
        
		return sb.toString();
	}
	
	
	/**
	 * Essenzialmente codifica un tags del core sFix in un tag del framework db di supporto
	 * @param _search [in] Il risultato di una ricerca fatta al database, sul quale si vuole estrapolare
	 * 						un tag
	 * @param _tag [in]		Il tag che si vuole estrapolare
	 * @return Il tag (in forma stringa) che si vuole estrapolare
	 * @throws IOException In caso non si riesca ad aprire un buffer in caso si voglia ricavare l'immagine di
	 * 			copertina
	 */
	public static String GetTagFromGNsearch(GNSearchResponse _search, final FixAudioFile.FIELD_tag _tag) throws IOException{
		if(_search==null) return new String("");
		switch(_tag){
		case TAG_ALBUM:
			return _search.getAlbumTitle();
		case TAG_ANNO:
			return _search.getAlbumReleaseYear();
		case TAG_ARTISTA:
			return _search.getArtist();
		case TAG_ARTISTA_ALBUM:
			return _search.getAlbumArtist();
		case TAG_COVER_IMAGE:
			if(_search.getCoverArt()==null) return new String("");
			if(_search.getCoverArt().getUrl()==null) return new String("");
			URL link_image = new URL(_search.getCoverArt().getUrl());
			InputStream istream =new BufferedInputStream(link_image.openStream());
			byte[] buffer = new byte[4096];
			ByteArrayOutputStream ostream = new ByteArrayOutputStream();
			int byte_readed=0;
			while(-1!=(byte_readed=istream.read(buffer))){
				ostream.write(buffer, 0, byte_readed);
			}
			istream.close();
			String rslt=new String(Base64.encodeToString(ostream.toByteArray(),Base64.DEFAULT));
			ostream.close();
			return rslt;
		case TAG_GENERE:
			return _search.getTrackGenre()[0].getData();
		case TAG_NTRACK:
			return String.valueOf(_search.getTrackNumber());
		case TAG_TITOLO:
			return _search.getTrackTitle();
		}
		return new String("");
	}
	/*-------------------CORE DATA*/
	private CoreFunctions(){ }
	private static final String ID_GRACENOTE_APP = "11274240-14B4C3DE19A14407EBD7A591E659A1A5";
	private static GNConfig serverGraceNote = null;
	private static RegisterFix registro;
	
	private static class RisultatoAnalisi_FingerPrinting implements GNFingerprintResultReady{
		public void GNResultReady(GNFingerprintResult res) {
			dataResult = res.getFingerprintData();
			if(dataResult==null){
				Log.e("sFix@FingerPrinting", res.getErrMessage());
			}
			this.done=true;
		}
		
		public boolean Result_IsReady(){
			return this.done;
		}
		
		public String GetData(){
			return this.dataResult;
		}
		
		private String dataResult = null;
		private boolean done = false;
	}
	
	public static class RisultatoRicerca implements GNSearchResultReady{
		private GNSearchResponse bestRisposta = null;
		private GNSearchResponse[] allRisposte = null;
		private boolean done = false;
		private boolean fail = false;
		
		@Override
		public void GNResultReady(GNSearchResult res) {
			if(res.isFingerprintSearchNoMatchStatus()){
				fail=true;
				Log.e("sFix@FingerPrinting", "Nessuna corrispondenza trovata");
			}else{
				bestRisposta = res.getBestResponse();
				allRisposte = res.getResponses();
			}
			this.done=true;			
		}
		
		/**
		 * @return 'true' se la query  stata inviata completamente al database e quest ultimo ha risposto.
		 * 		'false' se ancora deve essere inviata/completata la query.
		 * 		E' consigliabile fare un ciclo di attesa attivo finch al query non  ready!
		 * 		Perch questo metodo non  bloccante!
		 * @throws Exception in caso non venga trovata alcuna corrispondenza col database
		 */
		public boolean Result_IsReady() throws Exception{
			if(this.fail==true){
				throw new Exception("Nessuna corrispondenza trovata all'interno del database");
			}
			return this.done;
		}
		
		/**
		 * Ritorna un tag generico secondo una label definita dal core interno dell'app
		 * @param _tag [in] Il tag di cui si vuole prendere il valore
		 * @return Il tag dalla MIGLIORE corrisponenza. Ritorna una stringa vuota in caso non ci siano
		 * 		corrispondenze.
		 * @throws IOException In caso di mancata connettivit per il download dell'immagine di copertina
		 */
		public String GetGenericTag(final FIELD_tag _tag) throws IOException{
			if(bestRisposta==null) return new String("");
			return CoreFunctions.GetTagFromGNsearch(bestRisposta, _tag);
		}
		
		/**
		 * @return Il titolo del miglior risultato trovato. In caso di no-match ritorna una stringa vuota
		 */
		public String GetTitle(){
			if(bestRisposta==null) return new String("");
			return this.bestRisposta.getTrackTitle();
		}
		
		/**
		 * @return L'artista-album del miglior risultato
		 */
		public String GetAlbumArtist(){
			if(bestRisposta==null) return new String("");
			return this.bestRisposta.getAlbumArtist();
		}
		
		/**
		 * @return L'artista del brano
		 */
		public String GetArtist(){
			if(bestRisposta==null) return new String("");
			return this.bestRisposta.getArtist();
		}
		
		/**
		 * @return il titolo dell'album
		 */
		public String GetAlbumTitle(){
			if(bestRisposta==null) return new String("");
			return this.bestRisposta.getAlbumTitle();
		}
		
		/**
		 * @return La copertina dell'album
		 * @throws IOException In caso di mancata connettivit ad internet
		 */
		public byte[] GetCoverAlbum() throws IOException{
			if(bestRisposta==null) return null;
			if(bestRisposta.getCoverArt() == null) return null;
			if(bestRisposta.getCoverArt().getData()==null){
				if(bestRisposta.getCoverArt().getUrl()==null) return null;
				URL link_image = new URL(bestRisposta.getCoverArt().getUrl());
				InputStream istream = new BufferedInputStream(link_image.openStream());
				byte[] buffer = new byte[4096];
				ByteArrayOutputStream ostream = new ByteArrayOutputStream();
				int byte_readed=0;
				while((byte_readed = istream.read(buffer)) > 0){
					ostream.write(buffer, 0, byte_readed);
				}
				istream.close();
				return ostream.toByteArray();
			}
			return this.bestRisposta.getCoverArt().getData();
		}
		
		/**
		 * @return il tipo di dato multimediale della copertina (formato immagine)
		 */
		public String GetCoverAlbumMIMEtype(){
			if(bestRisposta==null) return new String("");
			if(bestRisposta.getCoverArt() == null ) return new String("");
			String rts = this.bestRisposta.getCoverArt().getMimeType();
			if(rts==null) return new String("image/jpeg");
			return rts;
		}
		
		/**
		 * @return il numero di quante corrispondenze sono state trovate come risposta.
		 * I metodi Get[*] accedono al miglior risultato.
		 * Per avere tutte le risposte complete utilizzare l'apposito metodo di classe
		 */
		public int Get_nResponses(){
			if(allRisposte==null) return 0;
			return allRisposte.length;
		}
	}
}
