/*
 * 			FIX_AUDIO_FILE_class
 * 				dev Neo333
 * 		Astrazione di un file musicale per la modifica dei suoi metadati
 * 
 * 				PROJECT: SOUND_FIX
 * 					dev BGG
 * */
package com.bgg.sfix.LogicApp;

import java.io.File;

import android.util.Base64;

import com.mpatric.mp3agic.ID3v2;
import com.mpatric.mp3agic.ID3v24Tag;
import com.mpatric.mp3agic.Mp3File;



public class FixAudioFile {
	private Mp3File aFile;
	private ID3v2 tags;
	private String filename_oring;
	
	
	/** Costruttore ed inizializzatore del file.
	 * @param[in] filename	Nome del file (comprensivo di path) di cui si vogliono leggere/modificare i tag
	 * 						musicali.
	 * @throws	Un'eccezione in caso il file non viene trovato oppure l'applicazione non ha i permessi di scrittura
	 * */
	public FixAudioFile(String filename) throws Exception{
		filename_oring=filename;
		aFile = new Mp3File(filename);
		if(aFile.hasId3v2Tag()){
			tags=aFile.getId3v2Tag();
		}else{
			tags=new ID3v24Tag();
			aFile.setId3v2Tag(tags);
		}
	}
	
	/**
	 * @return Il nome (comprensivo di path all'interno del filesystem) del file audio rappresentativo di questo oggetto
	 */
	public String GetAbsoluteFileName(){
		return this.filename_oring;
	}
	
	
	/**
	 * @return Il titolo del brano musicale. 
	 * 			NB. Ritorna una stringa vuota se non  presente!
	 * */
	public String GetTitle(){
		if(tags==null) return new String("");
		String rts=tags.getTitle();
		if(rts==null) return new String("");
		return rts;
	}
	
	
	/**
	 * @return Il nome dell'album del brano musicale.
	 * 			NB. Ritorna una stringa vuota se non  pesente il tag!
	 * */
	public String GetAlbum(){
		if(tags==null) return new String("");
		String rts=tags.getAlbum();
		if(rts==null) return new String("");
		return rts;
	}
	
	
	
	/**
	 * @return Il nome dell'artista dell'album.
	 * */
	public String GetAlbumArtist(){
		if(tags==null) return new String("");
		String rts=tags.getAlbumArtist();
		if(rts==null) return new String("");
		return rts;
	}
	
	/**
	 * @return L'artista/i della traccia audio
	 */
	public String GetArtist(){
		if(tags==null) return new String("");
		String rts = tags.getArtist();
		if(rts==null) return new String("");
		return rts;
	}
	
	/**
	 * @return L'anno di produzione del brano musicale
	 * @throws ID3Exception
	 */
	public int GetTrackNumber(){
		if(tags==null) return 0;
		String rts = tags.getTrack();
		if(rts==null) return 0;
		return Integer.valueOf(rts);
	}
	
	
	/**
	 * @return Il nome del genere del brano musicale
	 * */
	public int GetGenre(){
		if(tags==null) return 0;
		return tags.getGenre();
	}
	
	
	
	/**
	 * @return L'anno del brano musicale
	 * @throws ID3Exception 
	 * */
	public String GetYear(){
		if(tags==null) return new String("");
		String rts = tags.getYear();
		if(rts==null) return new String("");
		return rts;
	}
	
	/**
	 * @return I byte_data della copertina dell'immagine. Ritorna 'null' in caso non sia presente
	 */
	public byte[] GetCoverAlbum(){
		if(tags==null) return null;
		return tags.getAlbumImage();
	}
	
	
	/**
	 * Setta il titolo del brano musicale.
	 * @param [in] value	Il titolo del brano musicale col quale etichettare il brano
	 * @throws ID3Exception 
	 * @note Questa funzione non modifica fisicamente il file! Per effettuare quest'azione  necessario
	 * 		richiamare il metodo {@link FixAudioFile::Commit()}.
	 * @see {@link FixAudioFile::Commit()}
	 */
	public void SetTitle(String value){
		if(value!=null) tags.setTitle(value);
	}
	
	
	/**
	 * Setta il titolo dell'album musicale.
	 * @param [in] value	Il titolo dell'album musicale col quale etichettare il brano
	 * @throws ID3Exception 
	 * @note Questa funzione non modifica fisicamente il file! Per effettuare quest'azione  necessario
	 * 		richiamare il metodo {@link FixAudioFile::Commit()}.
	 * @see {@link FixAudioFile::Commit()}
	 */
	public void SetAlbum(String value){
		if(value!=null) tags.setAlbum(value);
	}
	
	
	/**
	 * Setta il nome dell'arista dell'album
	 * @param value Nome artista
	 * @throws ID3Exception 
	 */
	public void SetAlbumArtist(String value){
		if(value!=null) tags.setAlbumArtist(value);
	}
	
	/**
	 * Setta il nome dell'artista del brano musicale
	 * @param value [in] Nome artista
	 */
	public void SetArtist(String value){
		if(value!=null) tags.setArtist(value);
	}
	
	/**
	 * Setta il nome del genere del brano musicale
	 * @param value Nome del genere
	 * @throws ID3Exception 
	 */
	public void SetGenre(int value){
		tags.setGenre(value);
	}
	
	
	/**
	 * Setta l'anno di uscita del brano musicale
	 * @param value Anno di uscita
	 * @throws ID3Exception 
	 * @throws NumberFormatException 
	 */
	public void SetYear(String value){
		if(value!=null) tags.setYear(value);
	}
	
	
	/**
	 * Setta il numero di traccia all'interno del cd del brano musicale
	 * @param value Numero traccia
	 * @throws ID3Exception 
	 */
	public void SetTrackNumber(int value){
		tags.setTrack(String.valueOf(value));
	}
	
	/**
	 * Setta la copertina dell'album
	 * @param data [in]		Byte_data dell'immagine
	 * @param _mimetype		MimeType immagine (es. imge/jpeg)
	 */
	public void SetCoverAlbum(byte[] data, String _mimetype){
		if(data==null || _mimetype==null) return;
		tags.setAlbumImage(data, _mimetype);
	}
	
	
	
	/**
	 * Esegue il salvataggio delle modifiche ai tags fatti con gli specifici metodi della classe.
	 * @throws Exception	In caso l'operazione di salvataggio non vada a buon fine!
	 */
	public void Commit() throws Exception{
		if(aFile==null){
			if(filename_oring!=null){
				aFile=new Mp3File(filename_oring);
			}else{
				throw new Exception("Impossibile leggere il file mp3, nessun riferimento!");
			}
		}
		aFile.setId3v2Tag(tags);
		final String filename = aFile.getFilename();
		
		File _temp = new File(filename + ".sfix.temp");
		if(_temp.exists()==true){
			_temp.delete();
		}
		aFile.save(_temp.getAbsolutePath());
		aFile = null;
		
		File old = new File (filename);
		if((old.delete())==false){
			_temp.delete();
			throw new Exception("Impossibile completare il fixing del file. Un'altra applicazione potrebbe occuparlo!");
		}
		_temp.renameTo(old);
		_temp=null;
		old=null;
		
	}
	
	
	/*Set/Get Generici!*/
	enum FIELD_tag{
		TAG_TITOLO,
		TAG_ARTISTA,
		TAG_ARTISTA_ALBUM,
		TAG_ALBUM,
		TAG_GENERE,
		TAG_ANNO,
		TAG_NTRACK,
		TAG_COVER_IMAGE
	}
	public String GetATag(FIELD_tag _value) throws Exception{
		switch(_value){
		case TAG_ALBUM:
			return this.GetAlbum();
		case TAG_ANNO:
			return this.GetYear();
		case TAG_ARTISTA:
			return this.GetArtist();
		case TAG_GENERE:
			return String.valueOf(this.GetGenre());
		case TAG_NTRACK:
			return String.valueOf(GetTrackNumber());
		case TAG_TITOLO:
			return this.GetTitle();
		case TAG_ARTISTA_ALBUM:
			return this.GetAlbumArtist();
		case TAG_COVER_IMAGE:
			if(this.GetCoverAlbum()==null) return new String("");
			return new String(this.GetCoverAlbum());
		default:
			throw new Exception("@FixAudioFile:GetATag > Impossibile ricavare l'etichetta del tag!" +
					"Contattare l'amministratore del software");
		}
	}
	
	/**
	 * Setta un tag generico sul file musicale
	 * @param _tag [in] Il tag che si vuole modificare (dall'enumerazione)
	 * @param _value [in] Il valore del tag.
	 * @param _mimetype [in] Il tipo di dato multimediale.
	 * 						Questo serve solo nel caso si voglia modificare il tag relativo alla copertina
	 * 						dell'album. Negli altri casi si pu lasciare a null
	 * @throws Exception
	 */
	public void SetATag(FIELD_tag _tag, String _value, String _mimetype) throws Exception{
		if(_value==null) return;
		if(_value.isEmpty()==true) return;
		switch(_tag){
		case TAG_ALBUM:
			this.SetAlbum(_value);
			break;
		case TAG_ANNO:
			this.SetYear(_value);
			break;
		case TAG_ARTISTA:
			this.SetArtist(_value);
			break;
		case TAG_GENERE:
			this.SetGenre(Integer.valueOf(_value));
			break;
		case TAG_NTRACK:
			this.SetTrackNumber(Integer.valueOf(_value));
			break;
		case TAG_TITOLO:
			this.SetTitle(_value);
			break;
		case TAG_ARTISTA_ALBUM:
			this.SetAlbumArtist(_value);
			break;
		case TAG_COVER_IMAGE:
			this.SetCoverAlbum(Base64.decode(_value,Base64.DEFAULT), _mimetype);
			break;
		default:
			throw new Exception("@FixAudioFile:SetATag > Impossibile ricavare l'etichetta del tag!" +
					"Contattare l'amministratore del software");
		}
	}
}
