package com.bgg.sfix.LogicApp;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.util.HashMap;

import android.content.Context;

/**
 * 			REGISTER_FIX
 * @author Biagio
 * 
 *		DESCRIZIONE:
 *	Questa classe rappresenta il file di registro dell'applicazione.
 *	Questo registro serve per poter archiviare le operazioni compiute sui files processati dall'applicazione.
 *	Ogni volta che un file verr modificato dovr essere salvato sul registro varie informazioni come:
 *		-Il Nome del file processato
 *		-La modalit di esecuzione dell'algoritmo (SCANSIONE_RAPIDA, ecc..)
 *		-La data in cui il file  stato processato
 *
 *	In questo modo sar possibile all'algoritmo di poter essere pi veloce, qual ora l'utente voglia eseguire
 *	pi scansioni sul suo dispositivo, evitando di processare i files che sono stati gi elaborati in passato
 *	(o con una data recente)
 */
public class RegisterFix {
	/**
	 * Costruttore.
	 * Alla costruzione viene letto il registro dell'applicazione. Se il registro non viene trovato viene creato
	 * uno nuovo.
	 * @param _cont [in]	Contesto dell'appicazione, serve per individuare la directory private dell'applicazione
	 * @throws Exception
	 */
	public RegisterFix(Context _cont) throws Exception{
		contesto=_cont;
		final String pathInternal = contesto.getFilesDir().getAbsolutePath() + "/";
		pFile = new File(pathInternal + RegisterNameFile);
		if(pFile.exists()==false){
			contesto.openFileOutput(RegisterNameFile, Context.MODE_PRIVATE);
			pFile = new File(pathInternal + RegisterNameFile);
			if(pFile.exists()==false){
				pFile=null;
				throw new Exception("Impossibile creare il registro di restoring.");
			}
		}
		
		GetAllData_fromFile();
	}
	
	/**
	 * Verifica se un file  stato analizzato di recente controllando il registro dell'applicazione
	 * (se  stato inizializzato con la funzione apposita)
	 * @param _opt [in]		Le opzioni di fixing dell'algoritmo.
	 * @param _file [in]	Il file che si vuole verificare
	 * @return Ritorna 'true' se dal registro risulta che il file passato nel parametro  stato analizzato di recente
	 * 		(in questo caso il processo di fixing protrebbe essere inutile), altrimenti 'false'
	 */
	public boolean File_IsRecentAnalize(OptionAlgorithm _opt, File _file){
		if(_file == null) return false;
		InfoRow infoFile = this.GetRestored(_file.getAbsolutePath());
		if(infoFile==null) return false;
		
		/*Se i metodi di scansione sono diversi allora non  recente*/
		if(_opt.methodFix.compareTo(infoFile.Get_TypeScan())!=0) return false;
		
		/*Confronto se i tempi di modifica coincidono*/
		if(_file.lastModified() != infoFile.Get_Date()) return false;
		
		return true;
	}
	
	/**
	 * Inserisce nel registro uno storico memorizzando una traccia che  stata appena analizzata!
	 * In questo modo sar possibile (tramite l'apposito metodo) sapere se una traccia  stata analizzata di recente
	 * o meno. In caso positivo  possibile saltare l'operazione di fixing perch con tutta probabilit non necessaria!
	 * @param afile [in]	Il file che  stato appena fixato dall'algoritmo di cui si vuole memorizzare l'effettivo fixing
	 * 						avvenuto
	 * @param _option [in]	Le opzioni con le quali  stato fixato il file
	 * @throws IOException	In caso ci sia un errore di accesso al file in lettura!
	 */
	public void InsertTrackFixed(File afile, OptionAlgorithm _option) throws IOException{
		if(afile==null || pFile==null) return;
		this.InsertData(afile.getAbsolutePath(), new InfoRow(_option.methodFix, afile.lastModified()));
	}
	

	
	
	
		/*@@@@@@@DATA@@@@@@@@@*/
	private static final String RegisterNameFile = "sfix.reg";
	private File pFile = null;
	private HashMap<String, InfoRow> data = new HashMap<String, RegisterFix.InfoRow>();
	private Context contesto = null;
	
	private void GetAllData_fromFile() throws IOException{
		if(pFile==null) return;
		BufferedReader rbuffer = new BufferedReader(new FileReader(pFile));
		String buffer;
		while((buffer=rbuffer.readLine())!=null){
			InfoRow _temp = new InfoRow(OptionAlgorithm.TipoScansione.valueOf(rbuffer.readLine()),Long.valueOf(rbuffer.readLine()));
			data.put(buffer, _temp);
		}
		rbuffer.close();
	}
	
	/**
	 * Esegue una ricerca (hash) con la chiave data nel parametro.
	 * Se il File viene trovato nel registro ritorna una serie di informazioni-
	 * @param filename [in] 	Nomefile (assoluto) da cercare nel registro
	 * @return Le informazioni riguardo i passati processi del file, ritorna 'null' in caso non viene trovato
	 * 			nessun elemento per quella chiave di ingresso
	 */
	private InfoRow GetRestored(String filename){
		return data.get(filename);
	}
	
	/**
	 * Inserisce un nuovo elemento nel registro, aggiornando il registro in Memoria centrale e quello su memoria 
	 * di massa
	 * @param filename [in]		Chiave. Nome del file (assoluto) da inserire
	 * @param _data [in]		Informazioni su qel file
	 * @throws IOException In caso non si riesca ad accedere la file in memoria di massa
	 */
	private void InsertData(String filename, InfoRow _data) throws IOException{
		if(pFile==null) return;
		BufferedWriter rbuffer = new BufferedWriter(new FileWriter(pFile,true));
		rbuffer.write(filename);
		rbuffer.newLine();
		rbuffer.write(_data.type_scan.name());
		rbuffer.newLine();
		rbuffer.write(String.valueOf(_data.time_last_mod));
		rbuffer.newLine();
		rbuffer.close();
		data.put(filename,_data);
	}
	
	/**
	 * 		INFO_ROW
	 * Data informativo.
	 * 	Contiene le informazioni di una determinata chiave del registro.
	 * 	Vedere le info dei vari metodi per capire che informazioni contiene
	 * @author Biagio
	 *
	 */
	static private class InfoRow{
		private OptionAlgorithm.TipoScansione type_scan;
		private long time_last_mod;
		
		/**
		 * Costruttore.
		 * @param _type [in]	Il tipo di scansione col quale  stato processato il file musicale
		 * @param _date [in]	La data di ultima modifica del file
		 */
		public InfoRow(OptionAlgorithm.TipoScansione _type, long _last_mod) {
			type_scan=_type;
			time_last_mod=_last_mod;
		}
		
		/**
		 * @return La modalit con la quale  stato processato il file musicale
		 */
		public OptionAlgorithm.TipoScansione Get_TypeScan(){
			return type_scan;
		}

		/**
		 * La data in cui  stato modificato l'ultima volta il file musicale
		 * @return
		 */
		public long Get_Date(){
			return time_last_mod;
		}
	}
}
