package com.bgg.sfix;

import java.io.File;
import java.util.concurrent.ExecutionException;

import com.bgg.sfix.LogicApp.CoreFunctions;
import com.bgg.sfix.LogicApp.OptionAlgorithm;
import com.bgg.sfix.LogicApp.OptionAlgorithm.TipoScansione;
import com.bgg.sfix.Tasks.TaskCountTotalFiles;

import android.net.ConnectivityManager;
import android.os.AsyncTask;
import android.os.Bundle;
import android.os.Environment;
import android.preference.PreferenceManager;
import android.support.v4.app.NotificationCompat;
import android.util.Log;
import android.view.View;
import android.widget.Button;
import android.widget.ProgressBar;
import android.widget.TextView;
import android.app.Activity;
import android.app.Notification;
import android.app.NotificationManager;
import android.app.PendingIntent;
import android.content.Context;
import android.content.Intent;
import android.content.SharedPreferences;

public class ProgressActivity extends Activity {
	private static final String TAG_LOG = "@ProgressActivity";
	private ProcessIntern ServiceAlgoritm = new ProcessIntern();

	private ProgressBar loadBar;
	private TextView PercText;
	private TextView FileText;
	private TextView DoToText;
	private Button mettiBackgroundButton;
	private Button cancelButton;
	private Button visReportButton;
	private boolean __DEBUG__ = false;
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.activity_progress);
		Graphic.setdefaultFont(this,android.R.id.content,getBaseContext());
		this.loadBar=(ProgressBar)(findViewById(R.id.progressBar1_ProcessActivity));
		this.PercText=(TextView)(findViewById(R.id.textViewPercentProcess_ProcessActivity));
		this.FileText=(TextView)(findViewById(R.id.textViewInfoFile_ProcessActivity));
		this.DoToText=(TextView)(findViewById(R.id.ProgressActivity_FileDaFare));
		this.mettiBackgroundButton = (Button) (findViewById(R.id.buttonMettiInBackground));
		mettiBackgroundButton.setOnClickListener(new View.OnClickListener() {
			@Override
			public void onClick(View v) {
				Intent i_home = new Intent(Intent.ACTION_MAIN);
				i_home.addCategory(Intent.CATEGORY_HOME);
				startActivity(i_home);
			}
		});
		this.cancelButton = (Button)(findViewById(R.id.buttonCancelOperation));
		this.cancelButton.setOnClickListener(new View.OnClickListener() {
			@Override
			public void onClick(View v) {
				finish();
			}
		});
		this.visReportButton = (Button)(findViewById(R.id.button_ProgressActivity_VisualizzaReport));
		
		this.SetPercentProgressComplete(0);
		this.SetNameFileProcess("Inizializzazione ...");
	
		/*Ricavo le opzioni dell'utente di specifica per l'algoritmo*/
		OptionAlgorithm opzioni_alg = new OptionAlgorithm();
		
		SharedPreferences sharedPref = PreferenceManager.getDefaultSharedPreferences(this);

		opzioni_alg.ricorsive = sharedPref.getBoolean("ricorsione", true);
		opzioni_alg.modFileSystem = sharedPref.getBoolean("cartelle", false);
		
		sharedPref = getSharedPreferences("sfoglia", Activity.MODE_PRIVATE);
		opzioni_alg.root_toAnalyze = new File(sharedPref.getString("sfoglia",Environment.getExternalStorageDirectory().getAbsolutePath()));
		
		
		if(__DEBUG__==true){
			opzioni_alg.methodFix = TipoScansione.SCANSIONE_DEBUG;
		}else{
			switch(sharedPref.getInt("modalita", 1)){
			case 0:
				opzioni_alg.methodFix = TipoScansione.SCANSIONE_RAPIDA;
				break;
			case 1:
				opzioni_alg.methodFix = TipoScansione.SCANSIONE_COMPLETA;
				break;
			case 2:
				opzioni_alg.methodFix = TipoScansione.SCANSIONE_ESTESA;
				break;
			default:
				opzioni_alg.methodFix = TipoScansione.SCANSIONE_COMPLETA;
				break;
			}
		}
		
		opzioni_alg.downloadCovert = sharedPref.getBoolean("cover", true);
		/***************************************************************/
		
		ServiceAlgoritm.execute(opzioni_alg);
		
		Log.i(TAG_LOG,"Activity Creata con successo!");
	}
	
	@Override
	protected void onDestroy(){
		super.onDestroy();
		ServiceAlgoritm.CancelAllNotify();
		ServiceAlgoritm.cancel(true);
		Log.i(TAG_LOG,"Activity Distrutta con successo!");
	}
	
	
	
	
	
	
	

	/**
	 * Setta La percentuale di completamento della GUI
	 * @param value [in]	Valore in percentuale
	 */
	public void SetPercentProgressComplete(int value){
		this.loadBar.setProgress(value);
		this.PercText.setText(value + " %");
	}
	
	
	
	
	/**
	 * Setta il nome descrittivo del files processato correntemente
	 * @param value [in]	Nome del file processato
	 */
	public void SetNameFileProcess(String value){
		this.FileText.setText(value);
	}
	
	/**
	 * Setta il numero di file totali da processare sulla GUI
	 * @param value [in]	Numero di files totali da processare
	 */
	public void SetNumberTotalFile(int value){
		this.DoToText.setText("File Totali: "+String.valueOf(value));
	}

	/**
	 * Starta l'activity di result. Dovrebbe essere invocata a processo di algoritmo completo!
	 * @param _result [in] Un'instanza contenente i dati del processo da passare all'activity di riepilogo
	 * chiude la progress activity
	 */
	public void RunReportActivity(ResultData _result){
		Intent pass_param = new Intent(getApplicationContext(), ReportActivity.class);
		pass_param.putExtras(_result.GetBundle());
		startActivity(pass_param);
		finish ();
	}
	
	/**
	 * Costruisce un oggetto 'ResultData' a partire da un Bundle. Tipicamente questa funzione viene utilizzata
	 * per il passaggio di dati tra un'activity e l'altra.
	 * @param _data [in]	Il boundle dal quale estrarre i dati
	 * @return Un oggetto di tipo ResultData che contiene un riepilogo del risultato dell'operazione 
	 * 			dell'algoritmo di fixing
	 */
	public static final ResultData GetResultData_fromBundle(Bundle _data){
		ResultData rts = new ResultData();
		if(_data==null) return null;
		rts.done=_data.getInt(ResultData.ID_DONE);
		rts.errors=_data.getInt(ResultData.ID_ERRORS);
		rts.exception=_data.getString(ResultData.ID_MESS_ERR);
		rts.jump=_data.getInt(ResultData.ID_JUMP);
		return rts;
	}
	
	
	
	/**
	 * Controlla le condizione NECESSARIE per l'esecuzione dell'algoritmo.
	 * 		-Supporto di memoria accessibile in scrittura
	 * 		-Se il file indicato nel parametro esiste
	 * 		-Se c' connettivit (di qualsiasi genere)
	 * @param file_exists [in]	Path (absolute) di un file. Se questo file non esiste viene ritornata un'eccezione
	 * @throws Exception in caso una delle condizione NECESSARIE non siano verificate
	 */
	public void CheckPreConditions(String file_exists) throws Exception{
		String state = Environment.getExternalStorageState();
		if (Environment.MEDIA_MOUNTED.equals(state)==false){
			throw new Exception("Impossibile accedere al percorso indicato. Verificare i permessi dell'applicazione!");
		}
		if(new File(file_exists).exists()==false){
			throw new Exception("Impossibile accedere al percorso indicato. Verificare il percorso!");
		}
		ConnectivityManager conmng=(ConnectivityManager) getSystemService(Context.CONNECTIVITY_SERVICE);
		if(conmng.getActiveNetworkInfo().isAvailable()==false){
			throw new Exception("Impossibile accedere alla connessione internet. Verificare i permessi dell'applicazione e la connettivit");
		}
	}
	
	public void LaunchNotification_ScanComplete(ResultData _result){
		NotificationManager ntmng = (NotificationManager) getSystemService(NOTIFICATION_SERVICE);
		Notification notify = new Notification(R.drawable.ic_launcher, "SoundFix Scansione Completata!", System.currentTimeMillis());
		Intent myIntent = new Intent(this, ReportActivity.class);
		myIntent.putExtras(_result.GetBundle());
		PendingIntent pMyIntent = PendingIntent.getActivity(this, 0, myIntent, PendingIntent.FLAG_UPDATE_CURRENT);
		notify.setLatestEventInfo(getApplicationContext(), "Scansione SoundFix Completata!", 
				"La scansione dei brani musicali  stata completata con successo", pMyIntent);
		notify.flags |= Notification.FLAG_AUTO_CANCEL;
		
		ntmng.notify(0, notify);
	}
	
	
	
	
	/*--------------------------------------------------------------*/
	private class ProgressData{
		String desc_operation = new String("");
		int percent_complete = 0;
		int total_to_process = 0;
	}
	static public class ResultData{
		private int done = 0;
		private int errors = 0;
		private int jump = 0;
		private String exception = new String("");
		
		public ResultData(){ }
		
		public ResultData(int _done, int _errors, int _jump, String _mess){
			done = _done;
			errors = _errors;
			exception = _mess;
			jump = _jump;
			if(exception==null){
				exception=new String("");
			}
		}
		public Bundle GetBundle(){
			Bundle rts = new Bundle();
			rts.putInt(ResultData.ID_DONE, this.done);
			rts.putInt(ResultData.ID_ERRORS, this.errors);
			rts.putString(ResultData.ID_MESS_ERR, this.exception);
			rts.putInt(ResultData.ID_JUMP, this.jump);
			return rts;
		}
		
		/**
		 * @return Il numero totale di files analizzati (sia saltati che files non identificati->TUTTI)
		 */
		public int getDone() {
			return done;
		}
		
		/**
		 * @return Il numero di files che il processo non  riuscito a fixare. Per mancata corrispondenza nel DB o perch illegibile.
		 * Se questo valore, invece,  pari a '-1', allora vuol dire che si  verificato un errore critico nel sistema e non  stato
		 * possibile analizzare alcun file. In questo caso mandare a video una descrizione dell'errore tramite l'apposito metodo
		 * @see getException()
		 */
		public int getErrors() {
			return errors;
		}
		
		/**
		 * @return La stringa che descrive l'errore CRITICO verificatosi nel sistema.
		 * NOTA BENE: Per verificare se si  generato un errore critico nel sistema controllare con l'apposita funzione membro
		 * @see getErrors()
		 */
		public String getException() {
			return exception;
		}
		
		/**
		 * @return il numero di files che sono stati saltati nel processo di elaborazione perch risultati come analizzati di recente
		 */
		public int getJumpedFiles(){
			return jump;
		}

		private static final String ID_DONE = "VAL_ID_DONE";
		private static final String ID_ERRORS = "VAL_ID_ERRORS";
		private static final String ID_MESS_ERR = "VAL_ID_MESS_ERRO";
		private static final String ID_JUMP = "VAL_ID_JUMP";
	}
	
	/*----------------------TASK INTERNO CHE ESEGUE IL CUORE DELL'ALGORITMO--------------------------------*/
	private class ProcessIntern extends AsyncTask<OptionAlgorithm, ProgressData, ResultData>{
		private int total_files_toDo = 0;
		private int done = 0;
		private int errors = 0;
		private int files_recent = 0;
		private boolean force_quit = false;
		private NotificationManager ntfManager = null;
		private NotificationCompat.Builder notify = null;
		
		private void UpDateProgress(String desc){
			ProgressData current = new ProgressData();
			current.desc_operation=desc;
			current.total_to_process=total_files_toDo;
			if(total_files_toDo!=0)
				current.percent_complete=(int)((float)(done*100)/(float)(total_files_toDo));
			else
				current.percent_complete=0;
			publishProgress(current);
		}
		
		@Override
		protected void onPreExecute(){
			errors=0;
			total_files_toDo=0;
			done=0;
			files_recent=0;
			force_quit=false;

			ProgressActivity.this.mettiBackgroundButton.setVisibility(View.VISIBLE);
			ProgressActivity.this.cancelButton.setVisibility(View.VISIBLE);
			ProgressActivity.this.visReportButton.setVisibility(View.INVISIBLE);
			
			Intent myIntent = new Intent(ProgressActivity.this,ProgressActivity.class);
			myIntent.setFlags(Intent.FLAG_ACTIVITY_REORDER_TO_FRONT);
			PendingIntent pMyIntent = PendingIntent.getActivity(ProgressActivity.this, 0, myIntent, 0);
			ntfManager = (NotificationManager) getSystemService(NOTIFICATION_SERVICE);
			notify = new NotificationCompat.Builder(ProgressActivity.this);
			notify.setContentTitle("Scansione SoundFIX");
			notify.setSmallIcon(R.drawable.ic_launcher);
			notify.setProgress(100, 0, true);
			notify.setContentText("Scansione in corso dei brani musicali...");
			notify.setAutoCancel(false);
			notify.setOngoing(true);
			notify.setContentIntent(pMyIntent);
			CancelAllNotify();
			Log.i(TAG_LOG,"TaskProcess@FixAlg > Inizializzato con successo!");
		}
		
		@Override
		protected ResultData doInBackground(OptionAlgorithm... params) {			
			ResultData rts= new ResultData();
			
			//Inizializzo credenziali DATABASE gracenote
			try {
				UpDateProgress("Connessione al Server ...");
				CoreFunctions.InitGraceNote_Library(getApplicationContext(),params[0]);
			} catch (Exception e) {
				rts.errors=-1;
				rts.exception=e.getMessage();
				return rts;
			}
			
			//Provo ad inizializzare il registro dell'applicazione
			try {
				CoreFunctions.InitRegisterFix(getApplicationContext());
			} catch (Exception e1) {
				Log.e(TAG_LOG,e1.getMessage());
			}
			
			//Verifico connettivit e permessi di lettura/scrittura
			try {
				UpDateProgress("Verifica Requisiti ...");
				ProgressActivity.this.CheckPreConditions(params[0].root_toAnalyze.getAbsolutePath());
			} catch (Exception e) {
				rts.errors=-1;
				rts.exception=e.getMessage();
				return rts;
			}
			
			//Eseguo una stima del numero di files totali da processare
			total_files_toDo=0;
			UpDateProgress("Calcolo stima in corso ...");
			TaskCountTotalFiles _taskCount = new TaskCountTotalFiles();
			_taskCount.execute(params[0]);
			try {
				total_files_toDo = _taskCount.get();
				UpDateProgress(null);
			} catch (InterruptedException e) {
				rts.errors=-1;
				rts.exception=e.getMessage();
				return rts;
			} catch (ExecutionException e) {
				rts.errors=-1;
				rts.exception=e.getMessage();
				return rts;
			}
			
			UpDateProgress("Avvio analisi in corso ...");
			AnalizeAndFix_Directory(params[0],params[0].root_toAnalyze);
			
			rts.done=this.done;
			rts.errors=this.errors;
			rts.jump=this.files_recent;
			return rts;
		}
		
		@Override
		protected void onProgressUpdate(ProgressData... progress){
			ProgressData data = progress[0];
			if(data.percent_complete!=-1){
				ProgressActivity.this.SetPercentProgressComplete(data.percent_complete);
			}
			if(data.desc_operation!=null){
				ProgressActivity.this.SetNameFileProcess(data.desc_operation);
			}
			if(data.total_to_process!=-1){
				ProgressActivity.this.SetNumberTotalFile(data.total_to_process);
			}
			notify.setProgress(100, data.percent_complete, true);
			ntfManager.notify(0, notify.build());
		}
		
		@Override
		 protected void onPostExecute(final ResultData result) {
			ProgressActivity.this.mettiBackgroundButton.setVisibility(View.INVISIBLE);
			ProgressActivity.this.cancelButton.setVisibility(View.INVISIBLE);
			ProgressActivity.this.visReportButton.setVisibility(View.VISIBLE);
			ProgressActivity.this.visReportButton.setOnClickListener(new View.OnClickListener() {
				@Override
				public void onClick(View v) {
					RunReportActivity(result);					
				}
			});
			if(result.getErrors()==-1){
				ProgressActivity.this.SetNameFileProcess("Errore critico!");
			}else{
				ProgressActivity.this.SetNameFileProcess("Operazione Completata con successo!");
			}
			ProgressActivity.this.SetPercentProgressComplete(100);
			CancelAllNotify();
			CoreFunctions.LaunchMediaScan(ProgressActivity.this);
			LaunchNotification_ScanComplete(result);
			Log.i(TAG_LOG,"TaskProcess@FixAlg > Terminato con successo!");
		}
		
		@Override
		protected void onCancelled (){
			force_quit=true;
			CancelAllNotify();
			CoreFunctions.LaunchMediaScan(ProgressActivity.this);
			Log.i(TAG_LOG,"TaskProcess@FixAlg > Terminato inaspettatamente!");
		}
		
		public void CancelAllNotify(){
			ntfManager.cancel(0);
		}
		
		/**
		 * Analizza e Fixa una cartella/file secondo le opzioni descritte nel parametro
		 * @param _option	[in]	Opzioni di lavoro del processo
		 * @param _directory_or_file [in]	Directory root da analizzare o file
		 * NOTA BENE: Questa funzione  bloccate! E non lancia eccezioni: tutto ci che non riesce ad
		 * analizzare viene saltato!
		 */
		private void AnalizeAndFix_Directory(OptionAlgorithm _option, File _directory_or_file){
			if(_directory_or_file.isFile()==true){
				try {
					UpDateProgress(_directory_or_file.getName());
					if(CoreFunctions.AnalizeAndFix_File(_option,_directory_or_file)==false){
						files_recent++;
					}
				} catch (Exception e) {
					errors+=1;
					Log.e(TAG_LOG, e.getMessage());
				}
				done+=1;
				UpDateProgress(null);
				return;
			}
			
			File[] list_file = _directory_or_file.listFiles();
			if(list_file==null) return;
			for(int i=0; i<list_file.length && force_quit==false; i++){
				if(CoreFunctions.IsFileAnalizable(list_file[i])==true){
					if(list_file[i].isFile()==false){
						if(_option.ricorsive==true){
							AnalizeAndFix_Directory(_option,list_file[i]);
						}
					}else{
						try {
							UpDateProgress(list_file[i].getName());
							if(CoreFunctions.AnalizeAndFix_File(_option, list_file[i])==false){
								files_recent++;
							}
						} catch (Exception e) {
							errors+=1;
							Log.e(TAG_LOG, e.getMessage());
						}
						done+=1;
						UpDateProgress(null);
					}
				}
			}
			if(force_quit==true){
				CancelAllNotify();
			}
		}
		
	}
}
